@@ -92,7 +92,7 @@ typename ArrowType::c_type* fill_data_bitmap(typename ArrowType::c_type* ptr,
 
     auto k = 0;
     for (auto j = 0; j < total_rows; ++j) {
-        if (arrow::BitUtil::GetBit(bitmap_data, j)) ptr[k++] = raw_values[j];
+        if (util::bit_util::GetBit(bitmap_data, j)) ptr[k++] = raw_values[j];
     }
     return ptr + k;
 }
@@ -114,7 +114,7 @@ typename ArrowType::c_type min(Array_ptr& a) {
         auto bitmap = a->null_bitmap_data();
         auto res = std::numeric_limits<CType>::infinity();
         for (auto i = 0; i < a->length(); ++i) {
-            if (arrow::BitUtil::GetBit(bitmap, i) && raw[i] < res) res = raw[i];
+            if (util::bit_util::GetBit(bitmap, i) && raw[i] < res) res = raw[i];
         }
         return res;
     }
@@ -140,7 +140,7 @@ typename ArrowType::c_type max(Array_ptr& a) {
         auto bitmap = a->null_bitmap_data();
         auto res = -std::numeric_limits<CType>::infinity();
         for (auto i = 0; i < a->length(); ++i) {
-            if (arrow::BitUtil::GetBit(bitmap, i) && raw[i] > res) res = raw[i];
+            if (util::bit_util::GetBit(bitmap, i) && raw[i] > res) res = raw[i];
         }
         return res;
     }
@@ -169,7 +169,7 @@ typename ArrowType::c_type mean(const Buffer_ptr& bitmap, Array_ptr& a) {
     auto bitmap_data = bitmap->data();
     CType res = 0;
     for (auto i = 0; i < a->length(); ++i) {
-        if (arrow::BitUtil::GetBit(bitmap_data, i)) res += raw[i];
+        if (util::bit_util::GetBit(bitmap_data, i)) res += raw[i];
     }
 
     return res / static_cast<CType>(util::bit_util::non_null_count(bitmap, a->length()));
@@ -388,20 +388,20 @@ EigenMatrix<ArrowType> cov(Array_iterator begin, Array_iterator end) {
 }
 
 template <typename ArrowType>
-EigenMatrix<ArrowType> cov(Buffer_ptr bitmap, Array_ptr col) {
+typename ArrowType::c_type cov(Buffer_ptr bitmap, Array_ptr col) {
     using EigenVector = Matrix<typename ArrowType::c_type, Dynamic, 1>;
     std::vector<EigenVector> columns;
     columns.reserve(1);
 
-    auto c = to_eigen<false, ArrowType>(col, bitmap);
+    auto c = to_eigen<false, ArrowType>(bitmap, col);
     auto m = c->mean();
     columns.push_back(c->array() - m);
 
-    return compute_cov<ArrowType>(columns);
+    return *compute_cov<ArrowType>(columns)->data();
 }
 
 template <typename ArrowType, bool contains_null>
-EigenMatrix<ArrowType> cov(Array_ptr col) {
+typename ArrowType::c_type cov(Array_ptr col) {
     if constexpr (contains_null) {
         auto bitmap = col->null_bitmap();
         return cov<ArrowType>(bitmap, col);
@@ -414,7 +414,7 @@ EigenMatrix<ArrowType> cov(Array_ptr col) {
         auto m = c->mean();
         columns.push_back(c->array() - m);
 
-        return compute_cov<ArrowType>(columns);
+        return *compute_cov<ArrowType>(columns)->data();
     }
 }
 
@@ -478,7 +478,7 @@ EigenMatrix<ArrowType> sse(Buffer_ptr bitmap, Array_ptr col) {
     std::vector<EigenVector> columns;
     columns.reserve(1);
 
-    auto c = to_eigen<false, ArrowType>(col, bitmap);
+    auto c = to_eigen<false, ArrowType>(bitmap, col);
     auto m = c->mean();
     columns.push_back(c->array() - m);
 
@@ -1510,11 +1510,11 @@ public:
     }
 
     template <typename ArrowType, bool contains_null, typename Index, enable_if_index_t<Index, int> = 0>
-    EigenMatrix<ArrowType> cov(const Index& index) const {
+    typename ArrowType::c_type cov(const Index& index) const {
         return dataset::cov<ArrowType, contains_null>(derived().col(index));
     }
     template <typename ArrowType, typename Index, enable_if_index_t<Index, int> = 0>
-    EigenMatrix<ArrowType> cov(const Index& index) const {
+    typename ArrowType::c_type cov(const Index& index) const {
         if (null_count(index) == 0) {
             return cov<ArrowType, false>(index);
         } else {
@@ -1522,7 +1522,7 @@ public:
         }
     }
     template <typename ArrowType, typename Index, enable_if_index_t<Index, int> = 0>
-    EigenMatrix<ArrowType> cov(const Buffer_ptr& bitmap, const Index& index) const {
+    typename ArrowType::c_type cov(const Buffer_ptr& bitmap, const Index& index) const {
         return dataset::cov<ArrowType>(bitmap, derived().col(index));
     }
 
@@ -2052,7 +2052,7 @@ public:
             RAISE_STATUS_ERROR(builder.Reserve(valid_rows));
 
             for (auto i = 0; i < total_rows; ++i) {
-                if (arrow::BitUtil::GetBit(bitmap_data, i)) RAISE_STATUS_ERROR(builder.Append(i));
+                if (util::bit_util::GetBit(bitmap_data, i)) RAISE_STATUS_ERROR(builder.Append(i));
             }
 
             Array_ptr take_ind;
