@@ -34,26 +34,30 @@ namespace factors::continuous {
 
 std::shared_ptr<Factor> LinearGaussianCPDType::new_factor(const BayesianNetworkBase& m,
                                                           const std::string& variable,
-                                                          const std::vector<std::string>& evidence) const {
+                                                          const std::vector<std::string>& evidence,
+                                                          py::args args,
+                                                          py::kwargs kwargs) const {
     for (const auto& e : evidence) {
         if (m.node_type(e) == DiscreteFactorType::get()) {
-            return std::make_shared<CLinearGaussianCPD>(variable, evidence);
+            return generic_new_factor<CLinearGaussianCPD>(variable, evidence, args, kwargs);
         }
     }
 
-    return std::make_shared<LinearGaussianCPD>(variable, evidence);
+    return generic_new_factor<LinearGaussianCPD>(variable, evidence, args, kwargs);
 }
 
 std::shared_ptr<Factor> LinearGaussianCPDType::new_factor(const ConditionalBayesianNetworkBase& m,
                                                           const std::string& variable,
-                                                          const std::vector<std::string>& evidence) const {
+                                                          const std::vector<std::string>& evidence,
+                                                          py::args args,
+                                                          py::kwargs kwargs) const {
     for (const auto& e : evidence) {
         if (m.node_type(e) == DiscreteFactorType::get()) {
-            return std::make_shared<CLinearGaussianCPD>(variable, evidence);
+            return generic_new_factor<CLinearGaussianCPD>(variable, evidence, args, kwargs);
         }
     }
 
-    return std::make_shared<LinearGaussianCPD>(variable, evidence);
+    return generic_new_factor<LinearGaussianCPD>(variable, evidence, args, kwargs);
 }
 
 LinearGaussianCPD::LinearGaussianCPD(std::string variable, std::vector<std::string> evidence)
@@ -65,12 +69,13 @@ LinearGaussianCPD::LinearGaussianCPD(std::string variable,
                                      double variance)
     : Factor(variable, evidence), m_fitted(true), m_variance(variance) {
     if (static_cast<size_t>(beta.rows()) != (evidence.size() + 1)) {
-        throw py::value_error("Wrong number of beta parameters. Beta vector size: " + std::to_string(beta.size()) +
-                              ". Expected beta vector size: " + std::to_string(evidence.size() + 1) + ".");
+        throw std::invalid_argument(
+            "Wrong number of beta parameters. Beta vector size: " + std::to_string(beta.size()) +
+            ". Expected beta vector size: " + std::to_string(evidence.size() + 1) + ".");
     }
 
     if (variance <= 0) {
-        throw py::value_error("Variance must be a positive value.");
+        throw std::invalid_argument("Variance must be a positive value.");
     }
 
     m_beta = beta;
@@ -130,7 +135,7 @@ Matrix<typename ArrowType::c_type, Dynamic, 1> logl_impl_null(const DataFrame& d
     auto logl_ptr = logl.data();
 
     for (int i = 0; i < df->num_rows(); ++i) {
-        if (!arrow::BitUtil::GetBit(bitmap_data, i)) logl_ptr[i] = util::nan<CType>;
+        if (!util::bit_util::GetBit(bitmap_data, i)) logl_ptr[i] = util::nan<CType>;
     }
 
     return logl;
@@ -159,7 +164,7 @@ double slogl_impl_null(const DataFrame& df,
 
     double accum = 0;
     for (int i = 0; i < df->num_rows(); ++i) {
-        if (arrow::BitUtil::GetBit(bitmap_data, i)) accum += logl_ptr[i];
+        if (util::bit_util::GetBit(bitmap_data, i)) accum += logl_ptr[i];
     }
 
     return accum;
@@ -219,7 +224,7 @@ Matrix<typename ArrowType::c_type, Dynamic, 1> cdf_impl_null(const DataFrame& df
     ArrayVecType t(df->num_rows());
     if (evidence.empty()) {
         for (int64_t i = 0, end = df->num_rows(); i < end; ++i) {
-            if (arrow::BitUtil::GetBit(bitmap_data, i))
+            if (util::bit_util::GetBit(bitmap_data, i))
                 t(i) = std::erfc((beta(0) - var_array(i)) * inv_std * one_div_root_two<CType>);
             else
                 t(i) = util::nan<CType>;
@@ -235,7 +240,7 @@ Matrix<typename ArrowType::c_type, Dynamic, 1> cdf_impl_null(const DataFrame& df
         }
 
         for (int64_t i = 0, k = 0, end = df->num_rows(); i < end; ++i) {
-            if (arrow::BitUtil::GetBit(bitmap_data, i))
+            if (util::bit_util::GetBit(bitmap_data, i))
                 t(i) = std::erfc((means(k++) - var_array(i)) * inv_std * one_div_root_two<CType>);
             else
                 t(i) = util::nan<CType>;
@@ -264,7 +269,7 @@ VectorXd LinearGaussianCPD::logl(const DataFrame& df) const {
             }
         }
         default:
-            throw py::value_error("Wrong data type to compute logl. (double) or (float) data is expected.");
+            throw std::invalid_argument("Wrong data type to compute logl. [double] or [float] data is expected.");
     }
 }
 
@@ -284,7 +289,7 @@ double LinearGaussianCPD::slogl(const DataFrame& df) const {
                 return slogl_impl_null<arrow::FloatType>(df, m_beta, m_variance, this->variable(), this->evidence());
         }
         default:
-            throw py::value_error("Wrong data type to compute slogl. (double) or (float) data is expected.");
+            throw std::invalid_argument("Wrong data type to compute slogl. [double] or [float] data is expected.");
     }
 }
 
@@ -307,7 +312,7 @@ VectorXd LinearGaussianCPD::cdf(const DataFrame& df) const {
             }
         }
         default:
-            throw py::value_error("Wrong data type to compute cdf. (double) or (float) data is expected.");
+            throw std::invalid_argument("Wrong data type to compute cdf. [double] or [float] data is expected.");
     }
 }
 
