@@ -25,7 +25,7 @@ class BaseFactorParametersImpl : public BaseFactorParameters {
 public:
     BaseFactorParametersImpl(Args... args) : m_args(args...) {
         static_assert(std::is_constructible_v<BaseFactor, const std::string&, const std::vector<std::string>&, Args...>,
-                      "BaseFactor can not be constructed with provided Args...");
+                      "BaseFactor cannot be constructed with provided Args...");
     }
 
     std::shared_ptr<Factor> initialize(const std::string& variable,
@@ -56,7 +56,7 @@ public:
     SpecificBaseFactorParameters(std::unordered_map<Assignment, std::tuple<Args...>, AssignmentHash>& args)
         : m_args(args) {
         static_assert(std::is_constructible_v<BaseFactor, const std::string&, const std::vector<std::string>&, Args...>,
-                      "BaseFactor can not be constructed with provided Args...");
+                      "BaseFactor cannot be constructed with provided Args...");
     }
 
     std::shared_ptr<Factor> initialize(const std::string& variable,
@@ -87,7 +87,7 @@ private:
     std::unordered_map<Assignment, std::tuple<Args...>, AssignmentHash> m_args;
 };
 
-template <typename BaseFactor, const char* FactorName>
+template <typename BaseFactor, typename BaseFitter, typename FactorName>
 class DiscreteAdaptator : public Factor {
 public:
     template <typename... CArgs>
@@ -140,8 +140,8 @@ public:
 
     py::tuple __getstate__() const override;
 
-    static DiscreteAdaptator<BaseFactor, FactorName> __setstate__(py::tuple& t);
-    static DiscreteAdaptator<BaseFactor, FactorName> __setstate__(py::tuple&& t) { return __setstate__(t); }
+    static DiscreteAdaptator<BaseFactor, BaseFitter, FactorName> __setstate__(py::tuple& t);
+    static DiscreteAdaptator<BaseFactor, BaseFitter, FactorName> __setstate__(py::tuple&& t) { return __setstate__(t); }
 
 private:
     void check_fitted() const;
@@ -161,13 +161,13 @@ private:
     std::vector<std::shared_ptr<Factor>> m_factors;
 };
 
-template <typename BaseFactor, const char* FactorName>
-void DiscreteAdaptator<BaseFactor, FactorName>::check_fitted() const {
+template <typename BaseFactor, typename BaseFitter, typename FactorName>
+void DiscreteAdaptator<BaseFactor, BaseFitter, FactorName>::check_fitted() const {
     if (!m_fitted) throw std::invalid_argument("Factor " + ToString() + " not fitted.");
 }
 
-template <typename BaseFactor, const char* FactorName>
-void DiscreteAdaptator<BaseFactor, FactorName>::check_equal_domain(const DataFrame& df) const {
+template <typename BaseFactor, typename BaseFitter, typename FactorName>
+void DiscreteAdaptator<BaseFactor, BaseFitter, FactorName>::check_equal_domain(const DataFrame& df) const {
     df.raise_has_columns(evidence());
 
     for (const auto& e : m_continuous_evidence) {
@@ -185,8 +185,8 @@ void DiscreteAdaptator<BaseFactor, FactorName>::check_equal_domain(const DataFra
     }
 }
 
-template <typename BaseFactor, const char* FactorName>
-void DiscreteAdaptator<BaseFactor, FactorName>::fit(const DataFrame& df) {
+template <typename BaseFactor, typename BaseFitter, typename FactorName>
+void DiscreteAdaptator<BaseFactor, BaseFitter, FactorName>::fit(const DataFrame& df) {
     std::vector<std::string> discrete_evidence, continuous_evidence;
 
     for (const auto& e : evidence()) {
@@ -220,6 +220,8 @@ void DiscreteAdaptator<BaseFactor, FactorName>::fit(const DataFrame& df) {
         m_discrete_values.reserve(m_discrete_evidence.size());
         for (auto it = m_discrete_evidence.begin(), end = m_discrete_evidence.end(); it != end; ++it) {
             auto dict_evidence = std::static_pointer_cast<arrow::DictionaryArray>(df.col(*it));
+
+            factors::discrete::check_is_string_dictionary(dict_evidence, *it);
             auto dict_evidence_values = std::static_pointer_cast<arrow::StringArray>(dict_evidence->dictionary());
 
             std::vector<std::string> ev;
@@ -234,21 +236,22 @@ void DiscreteAdaptator<BaseFactor, FactorName>::fit(const DataFrame& df) {
         auto num_factors = m_cardinality.prod();
         m_factors.reserve(num_factors);
 
-        auto slice_builders = discrete_slice_indices(df, m_discrete_evidence, m_strides, num_factors);
-
-        auto assignments = assignments_from_indices(m_discrete_evidence, m_discrete_values, m_cardinality, m_strides);
+        auto slices = discrete_slice_indices(df, m_discrete_evidence, m_strides, num_factors);
 
         for (auto i = 0; i < num_factors; ++i) {
-            if (slice_builders[i].length() > 0) {
-                Array_ptr take_indices;
-                RAISE_STATUS_ERROR(slice_builders[i].Finish(&take_indices));
+            if (slices[i]) {
+                auto assignment =
+                    Assignment::from_index(i, m_discrete_evidence, m_discrete_values, m_cardinality, m_strides);
 
-                auto factor = m_args->initialize(variable(), m_continuous_evidence, assignments[i]);
+                auto factor = m_args->initialize(variable(), m_continuous_evidence, assignment);
                 m_factors.push_back(std::move(factor));
 
                 if (!m_factors.back()->fitted()) {
-                    auto df_filtered = df.take(take_indices);
-                    m_factors.back()->fit(df_filtered);
+                    auto df_filtered = df.take(slices[i]);
+
+                    if (!BaseFitter::fit(m_factors.back(), df_filtered)) {
+                        m_factors.back() = nullptr;
+                    }
                 }
             } else {
                 m_factors.push_back(nullptr);
@@ -279,46 +282,28 @@ void logl_impl(const std::shared_ptr<Factor>& f, const DataFrame& df, const Arra
     }
 }
 
-template <typename BaseFactor, const char* FactorName>
-VectorXd DiscreteAdaptator<BaseFactor, FactorName>::logl(const DataFrame& df) const {
+template <typename BaseFactor, typename BaseFitter, typename FactorName>
+VectorXd DiscreteAdaptator<BaseFactor, BaseFitter, FactorName>::logl(const DataFrame& df) const {
     run_checks(df);
 
     if (m_discrete_evidence.empty()) {
         return m_factors[0]->logl(df);
     } else {
         auto num_factors = m_factors.size();
-        auto slice_builders = discrete_slice_indices(df, m_discrete_evidence, m_strides, num_factors);
+        auto slices = discrete_slice_indices(df, m_discrete_evidence, m_strides, num_factors);
 
         VectorXd res(df->num_rows());
 
         if (auto combined_bitmap = df.combined_bitmap(m_discrete_evidence)) {
             auto bitmap_data = combined_bitmap->data();
             for (auto i = 0; i < df->num_rows(); ++i) {
-                if (!arrow::BitUtil::GetBit(bitmap_data, i)) res(i) = util::nan<double>;
+                if (!util::bit_util::GetBit(bitmap_data, i)) res(i) = util::nan<double>;
             }
         }
 
         for (size_t i = 0; i < num_factors; ++i) {
-            if (slice_builders[i].length() > 0) {
-                Array_ptr take_indices;
-                RAISE_STATUS_ERROR(slice_builders[i].Finish(&take_indices));
-
-                switch (take_indices->type_id()) {
-                    case Type::INT8:
-                        logl_impl<arrow::Int8Type>(m_factors[i], df, take_indices, res);
-                        break;
-                    case Type::INT16:
-                        logl_impl<arrow::Int16Type>(m_factors[i], df, take_indices, res);
-                        break;
-                    case Type::INT32:
-                        logl_impl<arrow::Int32Type>(m_factors[i], df, take_indices, res);
-                        break;
-                    case Type::INT64:
-                        logl_impl<arrow::Int64Type>(m_factors[i], df, take_indices, res);
-                        break;
-                    default:
-                        throw std::invalid_argument("Unreachable code!");
-                }
+            if (slices[i]) {
+                logl_impl<arrow::Int32Type>(m_factors[i], df, slices[i], res);
             }
         }
 
@@ -326,27 +311,22 @@ VectorXd DiscreteAdaptator<BaseFactor, FactorName>::logl(const DataFrame& df) co
     }
 }
 
-template <typename BaseFactor, const char* FactorName>
-double DiscreteAdaptator<BaseFactor, FactorName>::slogl(const DataFrame& df) const {
+template <typename BaseFactor, typename BaseFitter, typename FactorName>
+double DiscreteAdaptator<BaseFactor, BaseFitter, FactorName>::slogl(const DataFrame& df) const {
     run_checks(df);
 
     if (m_discrete_evidence.empty()) {
         return m_factors[0]->slogl(df);
     } else {
         auto num_factors = m_factors.size();
-        auto slice_builders = discrete_slice_indices(df, m_discrete_evidence, m_strides, num_factors);
+        auto slices = discrete_slice_indices(df, m_discrete_evidence, m_strides, num_factors);
 
         double res = 0;
 
         for (size_t i = 0; i < num_factors; ++i) {
-            if (slice_builders[i].length() > 0) {
-                Array_ptr take_indices;
-                RAISE_STATUS_ERROR(slice_builders[i].Finish(&take_indices));
-
-                if (m_factors[i]) {
-                    auto df_filtered = df.take(take_indices);
-                    res += m_factors[i]->slogl(df_filtered);
-                }
+            if (slices[i] && m_factors[i]) {
+                auto df_filtered = df.take(slices[i]);
+                res += m_factors[i]->slogl(df_filtered);
             }
         }
 
@@ -354,36 +334,34 @@ double DiscreteAdaptator<BaseFactor, FactorName>::slogl(const DataFrame& df) con
     }
 }
 
-template <typename BaseFactor, const char* FactorName>
-std::shared_ptr<Factor> DiscreteAdaptator<BaseFactor, FactorName>::conditional_factor(Assignment& assignment) const {
+template <typename BaseFactor, typename BaseFitter, typename FactorName>
+std::shared_ptr<Factor> DiscreteAdaptator<BaseFactor, BaseFitter, FactorName>::conditional_factor(
+    Assignment& assignment) const {
     check_fitted();
     auto index = assignment.index(m_discrete_evidence, m_discrete_values, m_strides);
     return m_factors[index];
 }
 
-template <typename BaseFactor, const char* FactorName>
-std::string DiscreteAdaptator<BaseFactor, FactorName>::ToString() const {
+template <typename BaseFactor, typename BaseFitter, typename FactorName>
+std::string DiscreteAdaptator<BaseFactor, BaseFitter, FactorName>::ToString() const {
     std::stringstream ss;
 
     if (!this->evidence().empty()) {
         const auto& e = evidence();
-        ss << "[" << FactorName << "] P(" << this->variable() << " | " << e[0];
+        ss << "[" << FactorName::str << "] P(" << this->variable() << " | " << e[0];
         for (size_t i = 1; i < e.size(); ++i) {
             ss << ", " << e[i];
         }
 
         ss << ")";
     } else {
-        ss << "[" << FactorName << "] P(" << variable() << ")";
+        ss << "[" << FactorName::str << "] P(" << variable() << ")";
     }
 
     if (!m_discrete_evidence.empty()) {
         if (m_fitted) {
             ss << std::endl;
 
-            auto assignments =
-                assignments_from_indices(m_discrete_evidence, m_discrete_values, m_cardinality, m_strides);
-
             fort::char_table table;
             table.set_cell_text_align(fort::text_align::center);
 
@@ -402,8 +380,9 @@ std::string DiscreteAdaptator<BaseFactor, FactorName>::ToString() const {
             table << fort::endr << fort::header;
             table.range_write_ln(m_discrete_evidence.begin(), m_discrete_evidence.end());
 
-            for (size_t k = 0; k < assignments.size(); ++k) {
-                const auto& ass = assignments[k];
+            for (size_t k = 0, num_factors = m_cardinality.prod(); k < num_factors; ++k) {
+                auto ass = Assignment::from_index(k, m_discrete_evidence, m_discrete_values, m_cardinality, m_strides);
+
                 for (const auto& discrete_evidence : m_discrete_evidence) {
                     table << static_cast<std::string>(ass.value(discrete_evidence));
                 }
@@ -463,7 +442,7 @@ void sample_factor_impl(const std::shared_ptr<Factor>& f,
 }
 
 template <typename ResultArrowType>
-void sample_impl(std::vector<arrow::AdaptiveIntBuilder>& slice_builders,
+void sample_impl(std::vector<Array_ptr>& slice_builders,
                  const std::vector<std::shared_ptr<Factor>>& factors,
                  int n,
                  const DataFrame& evidence_values,
@@ -472,42 +451,17 @@ void sample_impl(std::vector<arrow::AdaptiveIntBuilder>& slice_builders,
     auto num_factors = factors.size();
 
     for (size_t i = 0; i < num_factors; ++i) {
-        if (slice_builders[i].length() > 0) {
-            Array_ptr take_indices;
-            RAISE_STATUS_ERROR(slice_builders[i].Finish(&take_indices));
-
-            switch (take_indices->type_id()) {
-                case Type::INT8: {
-                    sample_factor_impl<arrow::Int8Type, ResultArrowType>(
-                        factors[i], n, evidence_values, seed + i, take_indices, res);
-                    break;
-                }
-                case Type::INT16: {
-                    sample_factor_impl<arrow::Int16Type, ResultArrowType>(
-                        factors[i], n, evidence_values, seed + i, take_indices, res);
-                    break;
-                }
-                case Type::INT32: {
-                    sample_factor_impl<arrow::Int32Type, ResultArrowType>(
-                        factors[i], n, evidence_values, seed + i, take_indices, res);
-                    break;
-                }
-                case Type::INT64: {
-                    sample_factor_impl<arrow::Int64Type, ResultArrowType>(
-                        factors[i], n, evidence_values, seed + i, take_indices, res);
-                    break;
-                }
-                default:
-                    throw std::invalid_argument("Unreachable code!");
-            }
+        if (slice_builders[i]) {
+            sample_factor_impl<arrow::Int32Type, ResultArrowType>(
+                factors[i], n, evidence_values, seed + i, slice_builders[i], res);
         }
     }
 }
 
-template <typename BaseFactor, const char* FactorName>
-Array_ptr DiscreteAdaptator<BaseFactor, FactorName>::sample(int n,
-                                                            const DataFrame& evidence_values,
-                                                            unsigned int seed) const {
+template <typename BaseFactor, typename BaseFitter, typename FactorName>
+Array_ptr DiscreteAdaptator<BaseFactor, BaseFitter, FactorName>::sample(int n,
+                                                                        const DataFrame& evidence_values,
+                                                                        unsigned int seed) const {
     if (n < 0) {
         throw std::invalid_argument("n should be a non-negative number");
     }
@@ -523,7 +477,7 @@ Array_ptr DiscreteAdaptator<BaseFactor, FactorName>::sample(int n,
         return m_factors[0]->sample(n, evidence_values, seed);
     } else {
         auto num_factors = m_factors.size();
-        auto slice_builders = discrete_slice_indices(evidence_values, m_discrete_evidence, m_strides, num_factors);
+        auto slices = discrete_slice_indices(evidence_values, m_discrete_evidence, m_strides, num_factors);
 
         Array_ptr res;
 
@@ -533,7 +487,7 @@ Array_ptr DiscreteAdaptator<BaseFactor, FactorName>::sample(int n,
                 RAISE_STATUS_ERROR(builder.AppendEmptyValues(n));
                 RAISE_STATUS_ERROR(builder.Finish(&res));
 
-                sample_impl<arrow::DoubleType>(slice_builders, m_factors, n, evidence_values, seed, res);
+                sample_impl<arrow::DoubleType>(slices, m_factors, n, evidence_values, seed, res);
                 break;
             }
             case Type::FLOAT: {
@@ -541,7 +495,7 @@ Array_ptr DiscreteAdaptator<BaseFactor, FactorName>::sample(int n,
                 RAISE_STATUS_ERROR(builder.AppendEmptyValues(n));
                 RAISE_STATUS_ERROR(builder.Finish(&res));
 
-                sample_impl<arrow::FloatType>(slice_builders, m_factors, n, evidence_values, seed, res);
+                sample_impl<arrow::FloatType>(slices, m_factors, n, evidence_values, seed, res);
                 break;
             }
             default:
@@ -552,8 +506,8 @@ Array_ptr DiscreteAdaptator<BaseFactor, FactorName>::sample(int n,
     }
 }
 
-template <typename BaseFactor, const char* FactorName>
-py::tuple DiscreteAdaptator<BaseFactor, FactorName>::__getstate__() const {
+template <typename BaseFactor, typename BaseFitter, typename FactorName>
+py::tuple DiscreteAdaptator<BaseFactor, BaseFitter, FactorName>::__getstate__() const {
     return py::make_tuple(this->variable(),
                           this->evidence(),
                           m_args->__getstate__(),
@@ -566,19 +520,20 @@ py::tuple DiscreteAdaptator<BaseFactor, FactorName>::__getstate__() const {
                           m_factors);
 }
 
-template <typename BaseFactor, const char* FactorName>
-DiscreteAdaptator<BaseFactor, FactorName> DiscreteAdaptator<BaseFactor, FactorName>::__setstate__(py::tuple& t) {
+template <typename BaseFactor, typename BaseFitter, typename FactorName>
+DiscreteAdaptator<BaseFactor, BaseFitter, FactorName>
+DiscreteAdaptator<BaseFactor, BaseFitter, FactorName>::__setstate__(py::tuple& t) {
     auto pyargs = t[2].cast<py::tuple>();
     auto specific_args = pyargs[0].cast<bool>();
 
     auto args = py::module_::import("pickle").attr("loads")(pyargs[1]);
     auto res = [specific_args, &t, &args]() {
         if (specific_args)
-            return py::type::of<DiscreteAdaptator<BaseFactor, FactorName>>()(t[0], t[1], args)
-                .template cast<DiscreteAdaptator<BaseFactor, FactorName>>();
+            return py::type::of<DiscreteAdaptator<BaseFactor, BaseFitter, FactorName>>()(t[0], t[1], args)
+                .template cast<DiscreteAdaptator<BaseFactor, BaseFitter, FactorName>>();
         else
-            return py::type::of<DiscreteAdaptator<BaseFactor, FactorName>>()(t[0], t[1], *args)
-                .template cast<DiscreteAdaptator<BaseFactor, FactorName>>();
+            return py::type::of<DiscreteAdaptator<BaseFactor, BaseFitter, FactorName>>()(t[0], t[1], *args)
+                .template cast<DiscreteAdaptator<BaseFactor, BaseFitter, FactorName>>();
     }();
 
     res.m_fitted = t[3].cast<bool>();
@@ -589,6 +544,7 @@ DiscreteAdaptator<BaseFactor, FactorName> DiscreteAdaptator<BaseFactor, FactorNa
         res.m_continuous_evidence = t[6].cast<std::vector<std::string>>();
         res.m_cardinality = t[7].cast<VectorXi>();
         res.m_strides = t[8].cast<VectorXi>();
+        // All the base factors are C++, so no need to keep Python object alive
         res.m_factors = t[9].cast<std::vector<std::shared_ptr<Factor>>>();
     }
 
