@@ -5,14 +5,15 @@ namespace learning::scores {
 double CVLikelihood::local_score(const BayesianNetworkBase& model,
                                  const std::string& variable,
                                  const std::vector<std::string>& evidence) const {
-    return local_score(model, *model.underlying_node_type(m_cv.data(), variable), variable, evidence);
+    return local_score(model, model.underlying_node_type(m_cv.data(), variable), variable, evidence);
 }
 
 double CVLikelihood::local_score(const BayesianNetworkBase& model,
-                                 const FactorType& variable_type,
+                                 const std::shared_ptr<FactorType>& variable_type,
                                  const std::string& variable,
                                  const std::vector<std::string>& evidence) const {
-    auto cpd = variable_type.new_factor(model, variable, evidence);
+    auto [args, kwargs] = m_arguments.args(variable, variable_type);
+    auto cpd = variable_type->new_factor(model, variable, evidence, args, kwargs);
 
     double loglik = 0;
     for (auto [train_df, test_df] : m_cv.loc(variable, evidence)) {
