@@ -4,6 +4,7 @@
 #include <random>
 #include <dataset/dataset.hpp>
 #include <factors/factors.hpp>
+#include <factors/arguments.hpp>
 #include <factors/unknown_factor.hpp>
 #include <graph/generic_graph.hpp>
 #include <util/parameter_traits.hpp>
@@ -11,7 +12,7 @@
 
 using arrow::DataType;
 using dataset::DataFrame;
-using factors::Factor, factors::UnknownFactorType;
+using factors::Factor, factors::Arguments, factors::UnknownFactorType;
 using graph::DagBase, graph::ConditionalDagBase, graph::Dag, graph::ConditionalDag;
 using util::ArcStringVector, util::FactorTypeVector;
 
@@ -76,7 +77,7 @@ public:
     virtual std::shared_ptr<Factor> cpd(const std::string& node) = 0;
     virtual const std::shared_ptr<Factor> cpd(const std::string& node) const = 0;
     virtual void add_cpds(const std::vector<std::shared_ptr<Factor>>& cpds) = 0;
-    virtual void fit(const DataFrame& df) = 0;
+    virtual void fit(const DataFrame& df, const Arguments& construction_args = Arguments()) = 0;
     virtual VectorXd logl(const DataFrame& df) const = 0;
     virtual double slogl(const DataFrame& df) const = 0;
     virtual std::shared_ptr<BayesianNetworkType> type() const = 0;
@@ -96,7 +97,8 @@ public:
     virtual std::shared_ptr<FactorType> underlying_node_type(const DataFrame&, const std::string& node) const = 0;
     virtual bool has_unknown_node_types() const = 0;
     virtual void set_node_type(const std::string& node, const std::shared_ptr<FactorType>& new_type) = 0;
-    virtual void set_unknown_node_types(const DataFrame& df) = 0;
+    virtual void set_unknown_node_types(const DataFrame& df,
+                                        const FactorTypeVector& type_blacklist = FactorTypeVector()) = 0;
     virtual void force_type_whitelist(const FactorTypeVector& type_whitelist) = 0;
     virtual std::string ToString() const = 0;
 
@@ -116,7 +118,18 @@ public:
         }
     }
 
-    static std::shared_ptr<BayesianNetworkBase> keep_python_alive(std::shared_ptr<BayesianNetworkBase>& m) {
+    static std::shared_ptr<BayesianNetworkBase>& keep_python_alive(std::shared_ptr<BayesianNetworkBase>& m) {
+        if (m && m->is_python_derived()) {
+            auto o = py::cast(m);
+            auto keep_python_state_alive = std::make_shared<py::object>(o);
+            auto ptr = o.cast<BayesianNetworkBase*>();
+            m = std::shared_ptr<BayesianNetworkBase>(keep_python_state_alive, ptr);
+        }
+
+        return m;
+    }
+
+    static std::shared_ptr<BayesianNetworkBase> keep_python_alive(const std::shared_ptr<BayesianNetworkBase>& m) {
         if (m && m->is_python_derived()) {
             auto o = py::cast(m);
             auto keep_python_state_alive = std::make_shared<py::object>(o);
@@ -140,6 +153,7 @@ public:
     virtual int num_joint_nodes() const = 0;
     virtual const std::vector<std::string>& interface_nodes() const = 0;
     virtual const std::vector<std::string>& joint_nodes() const = 0;
+    virtual ArcStringVector interface_arcs() const = 0;
     virtual int interface_collapsed_index(const std::string& name) const = 0;
     virtual int joint_collapsed_index(const std::string& name) const = 0;
     virtual const std::unordered_map<std::string, int>& interface_collapsed_indices() const = 0;
@@ -179,8 +193,20 @@ public:
         }
     }
 
-    static std::shared_ptr<ConditionalBayesianNetworkBase> keep_python_alive(
+    static std::shared_ptr<ConditionalBayesianNetworkBase>& keep_python_alive(
         std::shared_ptr<ConditionalBayesianNetworkBase>& m) {
+        if (m && m->is_python_derived()) {
+            auto o = py::cast(m);
+            auto keep_python_state_alive = std::make_shared<py::object>(o);
+            auto ptr = o.cast<ConditionalBayesianNetworkBase*>();
+            m = std::shared_ptr<ConditionalBayesianNetworkBase>(keep_python_state_alive, ptr);
+        }
+
+        return m;
+    }
+
+    static std::shared_ptr<ConditionalBayesianNetworkBase> keep_python_alive(
+        const std::shared_ptr<ConditionalBayesianNetworkBase>& m) {
         if (m && m->is_python_derived()) {
             auto o = py::cast(m);
             auto keep_python_state_alive = std::make_shared<py::object>(o);
@@ -205,7 +231,18 @@ public:
     virtual std::shared_ptr<ConditionalBayesianNetworkBase> new_cbn(
         const std::vector<std::string>& nodes, const std::vector<std::string>& interface_nodes) const = 0;
 
-    static std::shared_ptr<BayesianNetworkType> keep_python_alive(std::shared_ptr<BayesianNetworkType>& s) {
+    static std::shared_ptr<BayesianNetworkType>& keep_python_alive(std::shared_ptr<BayesianNetworkType>& s) {
+        if (s && s->is_python_derived()) {
+            auto o = py::cast(s);
+            auto keep_python_state_alive = std::make_shared<py::object>(o);
+            auto ptr = o.cast<BayesianNetworkType*>();
+            s = std::shared_ptr<BayesianNetworkType>(keep_python_state_alive, ptr);
+        }
+
+        return s;
+    }
+
+    static std::shared_ptr<BayesianNetworkType> keep_python_alive(const std::shared_ptr<BayesianNetworkType>& s) {
         if (s && s->is_python_derived()) {
             auto o = py::cast(s);
             auto keep_python_state_alive = std::make_shared<py::object>(o);
@@ -219,7 +256,8 @@ public:
     virtual bool is_homogeneous() const = 0;
 
     virtual std::shared_ptr<FactorType> default_node_type() const = 0;
-    virtual std::shared_ptr<FactorType> data_default_node_type(const std::shared_ptr<DataType>& dt) const = 0;
+    virtual std::vector<std::shared_ptr<FactorType>> data_default_node_type(
+        const std::shared_ptr<DataType>& dt) const = 0;
 
     virtual bool compatible_node_type(const BayesianNetworkBase&,
                                       const std::string&,
@@ -569,8 +607,8 @@ public:
         if (!m_cpds.empty() && m_cpds[idx])
             return m_cpds[idx];
         else
-            throw py::value_error("CPD of variable \"" + node +
-                                  "\" not added. Call add_cpds() or fit() to add the CPD.");
+            throw std::invalid_argument("CPD of variable \"" + node +
+                                        "\" not added. Call add_cpds() or fit() to add the CPD.");
     }
 
     const std::shared_ptr<Factor> cpd(const std::string& node) const override {
@@ -578,8 +616,8 @@ public:
         if (!m_cpds.empty() && m_cpds[idx])
             return m_cpds[idx];
         else
-            throw py::value_error("CPD of variable \"" + node +
-                                  "\" not added. Call add_cpds() or fit() to add the CPD.");
+            throw std::invalid_argument("CPD of variable \"" + node +
+                                        "\" not added. Call add_cpds() or fit() to add the CPD.");
     }
 
     virtual void check_compatible_cpd(const Factor& cpd) const;
@@ -587,7 +625,7 @@ public:
                                     const FactorType& model_node_type,
                                     const std::vector<std::string>& model_parents) const;
     void add_cpds(const std::vector<std::shared_ptr<Factor>>& cpds) override;
-    void fit(const DataFrame& df) override;
+    void fit(const DataFrame& df, const Arguments& construction_args = Arguments()) override;
     VectorXd logl(const DataFrame& df) const override;
     double slogl(const DataFrame& df) const override;
 
@@ -605,6 +643,9 @@ public:
     void save(std::string name, bool include_cpd = false) const override;
     virtual py::tuple __getstate__() const override;
 
+    template <typename DerivedBN>
+    friend std::shared_ptr<DerivedBN> __derived_bn_setstate__(py::tuple& t);
+
     bool include_cpd() const override { return m_include_cpd; }
 
     void set_include_cpd(bool include_cpd) const override { m_include_cpd = include_cpd; }
@@ -627,7 +668,14 @@ public:
             if (*m_node_types[node_index] != UnknownFactorType::get_ref()) {
                 return m_node_types[node_index];
             } else {
-                return m_type->data_default_node_type(df.col(node)->type());
+                auto nt = m_type->data_default_node_type(df.col(node)->type());
+                if (nt.empty()) {
+                    throw std::invalid_argument("There is no underlying FactorType for node " + node +
+                                                " as there is no valid FactorType for DataType " +
+                                                df.col(node)->type()->ToString());
+                } else {
+                    return nt[0];
+                }
             }
         }
     }
@@ -669,12 +717,31 @@ public:
         }
     }
 
-    void set_unknown_node_types(const DataFrame& df) override {
+    void set_unknown_node_types(const DataFrame& df,
+                                const FactorTypeVector& type_blacklist = FactorTypeVector()) override {
         if (m_type->is_homogeneous()) return;
 
+        util::FactorTypeSet blacklist_set{type_blacklist.begin(), type_blacklist.end()};
+
         FactorTypeVector new_types;
         for (const auto& nn : nodes()) {
-            new_types.emplace_back(nn, m_type->data_default_node_type(df.col(nn)->type()));
+            if (*node_type(nn) == UnknownFactorType::get_ref()) {
+                auto default_node_types = m_type->data_default_node_type(df.col(nn)->type());
+
+                bool found_new_type = false;
+                for (const auto& type : default_node_types) {
+                    auto new_pair = std::make_pair(nn, type);
+                    if (blacklist_set.count(new_pair) == 0) {
+                        new_types.push_back(std::move(new_pair));
+                        found_new_type = true;
+                        break;
+                    }
+                }
+
+                if (!found_new_type) {
+                    throw std::invalid_argument("A valid FactorType for node " + nn + " could not be inferred.");
+                }
+            }
         }
 
         force_type_whitelist(new_types);
@@ -890,7 +957,7 @@ bool BNGeneric<DagType>::must_construct_cpd(const Factor& cpd,
 }
 
 template <typename DagType>
-void BNGeneric<DagType>::fit(const DataFrame& df) {
+void BNGeneric<DagType>::fit(const DataFrame& df, const Arguments& construction_args) {
     if (m_cpds.empty()) {
         m_cpds.resize(num_raw_nodes());
     }
@@ -901,7 +968,7 @@ void BNGeneric<DagType>::fit(const DataFrame& df) {
             auto node_type_ = node_type(nn);
 
             if (*node_type_ == UnknownFactorType::get_ref()) {
-                new_factor_types.push_back({nn, m_type->data_default_node_type(df.col(nn)->type())});
+                new_factor_types.push_back({nn, underlying_node_type(df, nn)});
             }
         }
     }
@@ -917,7 +984,8 @@ void BNGeneric<DagType>::fit(const DataFrame& df) {
         auto node_type_ = node_type(nn);
 
         if (!m_cpds[i] || must_construct_cpd(*m_cpds[i], *node_type_, p)) {
-            m_cpds[i] = node_type_->new_factor(*this, nn, p);
+            auto [args, kwargs] = construction_args.args(nn, node_type_);
+            m_cpds[i] = node_type_->new_factor(*this, nn, p, args, kwargs);
             m_cpds[i]->fit(df);
         } else if (!m_cpds[i]->fitted()) {
             m_cpds[i]->fit(df);
@@ -962,7 +1030,7 @@ DataFrame BNGeneric<DagType>::sample(int n, unsigned int seed, bool ordered) con
     if constexpr (graph::is_conditional_graph_v<DagType>) {
         if (this->num_interface_nodes() > 0)
             throw std::runtime_error(
-                "Can not sample from ConditionalBayesianNetwork "
+                "ConditionalBayesianNetwork cannot sample "
                 "if evidence is not provided for the interface nodes.");
     }
 
@@ -1109,27 +1177,22 @@ void __nonderived_bn_setstate__(py::object& self, py::tuple& t) {
     auto type = t[1].cast<std::shared_ptr<BayesianNetworkType>>();
 
     // Initialize the C++ side:
+    // The Python constructor ensures that the python objects are kept alive.
     if (type->is_homogeneous()) {
         pybntype.attr("__init__")(self, type, std::move(dag));
     } else {
         auto node_types = t[2].cast<FactorTypeVector>();
         if (node_types.empty()) pybntype.attr("__init__")(self, type, std::move(dag));
 
-        if constexpr (std::is_constructible_v<BayesianNetwork,
-                                              std::shared_ptr<BayesianNetworkType>,
-                                              Dag&&,
-                                              FactorTypeVector>) {
-            pybntype.attr("__init__")(self, type, std::move(dag), node_types);
-        } else {
-            throw std::runtime_error("Invalid node types array for non-homogeneous Bayesian network.");
-        }
+        // BNType is BayesianNetwork or ConditionalBayesianNetwork, so this constructor always exists.
+        pybntype.attr("__init__")(self, type, std::move(dag), node_types);
     }
 
     auto cpp_self = self.cast<std::shared_ptr<BNType>>();
 
     if (t[3].cast<bool>()) {
         auto cpds = t[4].cast<std::vector<std::shared_ptr<Factor>>>();
-
+        Factor::keep_vector_python_alive(cpds);
         cpp_self->add_cpds(cpds);
     }
 }
@@ -1146,20 +1209,25 @@ std::shared_ptr<DerivedBN> __derived_bn_setstate__(py::tuple& t) {
         if (type->is_homogeneous()) {
             return std::make_shared<DerivedBN>(std::move(dag));
         } else {
+            // Only C++ FactorType are allowed in these Bayesian networks, so no need to keep Python alive.
             auto node_types = t[2].cast<FactorTypeVector>();
             if (node_types.empty()) return std::make_shared<DerivedBN>(std::move(dag));
 
             if constexpr (std::is_constructible_v<DerivedBN, DagType&&, FactorTypeVector>) {
                 return std::make_shared<DerivedBN>(std::move(dag), node_types);
             } else {
-                throw std::runtime_error("Invalid node types array for non-homogeneous Bayesian network.");
+                auto m = std::make_shared<DerivedBN>(std::move(dag));
+                m->initialize_types(node_types);
+                return m;
             }
         }
     }();
 
     if (t[3].cast<bool>()) {
         auto cpds = t[4].cast<std::vector<std::shared_ptr<Factor>>>();
-
+        // Keep Python factors alive. Only needed because someone could create a Python Factor which
+        // implements a C++-implemented FactorType.
+        Factor::keep_vector_python_alive(cpds);
         bn->add_cpds(cpds);
     }
 
@@ -1178,6 +1246,8 @@ public:
 
     const std::vector<std::string>& joint_nodes() const override { return this->g.joint_nodes(); }
 
+    ArcStringVector interface_arcs() const override { return this->g.interface_arcs(); }
+
     int interface_collapsed_index(const std::string& name) const override {
         return this->g.interface_collapsed_index(name);
     }
