@@ -2,11 +2,19 @@
 
 namespace models {
 
-MapDataToFactor keep_MapDataToFactor_alive(MapDataToFactor& m) {
+MapDataToFactor& keep_MapDataToFactor_alive(MapDataToFactor& m) {
+    for (auto& item : m) {
+        FactorType::keep_vector_python_alive(item.second);
+    }
+
+    return m;
+}
+
+MapDataToFactor keep_MapDataToFactor_alive(const MapDataToFactor& m) {
     MapDataToFactor alive;
 
     for (auto& item : m) {
-        alive.insert({item.first, FactorType::keep_python_alive(item.second)});
+        alive.insert({item.first, FactorType::keep_vector_python_alive(item.second)});
     }
 
     return alive;
