@@ -43,6 +43,7 @@ struct OpenCL_kernel_traits<arrow::DoubleType> {
     inline constexpr static const char* solve = "solve_double";
     inline constexpr static const char* square = "square_double";
     inline constexpr static const char* logl_values_1d_mat = "logl_values_1d_mat_double";
+    inline constexpr static const char* add_logl_values_1d_mat = "add_logl_values_1d_mat_double";
     inline constexpr static const char* substract = "substract_double";
     inline constexpr static const char* logl_values_mat_column = "logl_values_mat_column_double";
     inline constexpr static const char* logl_values_mat_row = "logl_values_mat_row_double";
@@ -53,13 +54,19 @@ struct OpenCL_kernel_traits<arrow::DoubleType> {
     inline constexpr static const char* add_accum_sum_mat_cols = "add_accum_sum_mat_cols_double";
     inline constexpr static const char* normalize_accum_sum_mat_cols = "normalize_accum_sum_mat_cols_double";
     inline constexpr static const char* find_random_indices = "find_random_indices_double";
-    inline constexpr static const char* conditional_means_1d = "conditional_means_1d_mat_double";
+    inline constexpr static const char* conditional_means_1d = "conditional_means_1d_double";
     inline constexpr static const char* conditional_means_column = "conditional_means_column_double";
     inline constexpr static const char* conditional_means_row = "conditional_means_row_double";
     inline constexpr static const char* univariate_normal_cdf = "univariate_normal_cdf_double";
     inline constexpr static const char* normal_cdf = "normal_cdf_double";
     inline constexpr static const char* product_elementwise = "product_elementwise_double";
     inline constexpr static const char* division_elementwise = "division_elementwise_double";
+    inline constexpr static const char* sum_ucv_1d = "sum_ucv_1d_double";
+    inline constexpr static const char* triangular_substract_mat = "triangular_substract_mat_double";
+    inline constexpr static const char* sum_ucv_mat = "sum_ucv_mat_double";
+    inline constexpr static const char* ucv_diag = "ucv_diag_double";
+    inline constexpr static const char* sum_ucv_diag = "sum_ucv_diag_double";
+    inline constexpr static const char* copy_ucv_diag = "copy_ucv_diag_double";
 };
 
 template <>
@@ -72,6 +79,7 @@ struct OpenCL_kernel_traits<arrow::FloatType> {
     inline constexpr static const char* solve = "solve_float";
     inline constexpr static const char* square = "square_float";
     inline constexpr static const char* logl_values_1d_mat = "logl_values_1d_mat_float";
+    inline constexpr static const char* add_logl_values_1d_mat = "add_logl_values_1d_mat_float";
     inline constexpr static const char* substract = "substract_float";
     inline constexpr static const char* logl_values_mat_column = "logl_values_mat_column_float";
     inline constexpr static const char* logl_values_mat_row = "logl_values_mat_row_float";
@@ -82,13 +90,19 @@ struct OpenCL_kernel_traits<arrow::FloatType> {
     inline constexpr static const char* add_accum_sum_mat_cols = "add_accum_sum_mat_cols_float";
     inline constexpr static const char* normalize_accum_sum_mat_cols = "normalize_accum_sum_mat_cols_float";
     inline constexpr static const char* find_random_indices = "find_random_indices_float";
-    inline constexpr static const char* conditional_means_1d = "conditional_means_1d_mat_float";
+    inline constexpr static const char* conditional_means_1d = "conditional_means_1d_float";
     inline constexpr static const char* conditional_means_column = "conditional_means_column_float";
     inline constexpr static const char* conditional_means_row = "conditional_means_row_float";
     inline constexpr static const char* univariate_normal_cdf = "univariate_normal_cdf_float";
     inline constexpr static const char* normal_cdf = "normal_cdf_float";
     inline constexpr static const char* product_elementwise = "product_elementwise_float";
     inline constexpr static const char* division_elementwise = "division_elementwise_float";
+    inline constexpr static const char* sum_ucv_1d = "sum_ucv_1d_float";
+    inline constexpr static const char* triangular_substract_mat = "triangular_substract_mat_float";
+    inline constexpr static const char* sum_ucv_mat = "sum_ucv_mat_float";
+    inline constexpr static const char* ucv_diag = "ucv_diag_float";
+    inline constexpr static const char* sum_ucv_diag = "sum_ucv_diag_float";
+    inline constexpr static const char* copy_ucv_diag = "copy_ucv_diag_float";
 };
 
 template <typename ArrowType>
@@ -132,6 +146,13 @@ public:
     template <typename T>
     void fill_buffer(cl::Buffer& b, const T value, unsigned int length);
 
+    template <typename ArrowType>
+    std::pair<cl::Buffer, uint64_t> allocate_temp_mat(size_t rows, size_t cols, size_t max_cols = 64) {
+        using CType = typename ArrowType::c_type;
+        auto allocated_m = std::min(cols, max_cols);
+        return std::make_pair(new_buffer<CType>(rows * allocated_m), allocated_m);
+    }
+
     cl::Kernel& kernel(const char* name);
     cl::CommandQueue& queue() { return m_queue; }
 
@@ -151,10 +172,6 @@ public:
         return output;
     }
 
-    template <typename ArrowType>
-    void sum1d(cl::Buffer& input_vec, int input_length, cl::Buffer& output_buffer, int output_offset) {
-        reduction1d<ArrowType, SumReduction<ArrowType>>(input_vec, input_length, output_buffer, output_offset);
-    }
 
     template <typename ArrowType, typename Reduction>
     cl::Buffer reduction_cols(const cl::Buffer& input_mat, int input_rows, int input_cols);
@@ -239,8 +256,8 @@ cl::Buffer OpenCLConfig::new_buffer(int size, cl_mem_flags flags) {
     cl::Buffer b(m_context, flags, sizeof(T) * size, NULL, &err_code);
 
     if (err_code != CL_SUCCESS) {
-        throw std::runtime_error(std::string("Error creating OpenCL buffer. ") + opencl::opencl_error(err_code) + " (" +
-                                 std::to_string(err_code) + ").");
+        throw std::runtime_error(std::string("Error creating OpenCL buffer of size ") + std::to_string(size) +
+                                 opencl::opencl_error(err_code) + " (" + std::to_string(err_code) + ").");
     }
 
     return b;
