@@ -4,6 +4,7 @@
 #include <pybind11/eigen.h>
 #include <pybind11/operators.h>
 #include <factors/factors.hpp>
+#include <factors/arguments.hpp>
 #include <factors/continuous/LinearGaussianCPD.hpp>
 #include <factors/continuous/CKDE.hpp>
 #include <factors/assignment.hpp>
@@ -14,18 +15,15 @@
 
 namespace py = pybind11;
 
+using factors::Arguments, factors::Args, factors::Kwargs;
 using factors::Factor, factors::continuous::LinearGaussianCPD, factors::continuous::CLinearGaussianCPD,
-    factors::continuous::KDE, factors::continuous::DCKDE, factors::continuous::CKDE;
+    factors::continuous::HCKDE, factors::continuous::CKDE, factors::discrete::DiscreteFactor;
 using factors::FactorType, factors::continuous::LinearGaussianCPDType, factors::continuous::CKDEType,
     factors::discrete::DiscreteFactorType;
 
-using factors::discrete::DiscreteFactor;
-
 using factors::Assignment, factors::AssignmentValue, factors::AssignmentHash;
-
-using util::random_seed_arg;
-
 using models::BayesianNetworkBase, models::ConditionalBayesianNetworkBase;
+using util::random_seed_arg;
 
 class PyFactorType : public FactorType {
 public:
@@ -39,19 +37,26 @@ public:
 
     std::shared_ptr<Factor> new_factor(const BayesianNetworkBase& model,
                                        const std::string& variable,
-                                       const std::vector<std::string>& parents) const override {
+                                       const std::vector<std::string>& parents,
+                                       py::args args = py::args{},
+                                       py::kwargs kwargs = py::kwargs{}) const override {
         pybind11::gil_scoped_acquire gil;
         pybind11::function override = pybind11::get_override(static_cast<const FactorType*>(this), "new_factor");
 
         if (override) {
-            auto o = override(model.shared_from_this(), variable, parents);
+            auto o = override(model.shared_from_this(), variable, parents, *args, **kwargs);
 
             if (o.is(py::none())) {
-                throw std::invalid_argument("FactorType::new_factor can not return None.");
+                throw std::invalid_argument("FactorType::new_factor cannot return None.");
             }
 
-            auto f = o.cast<std::shared_ptr<Factor>>();
-            return Factor::keep_python_alive(f);
+            try {
+                auto f = o.cast<std::shared_ptr<Factor>>();
+                Factor::keep_python_alive(f);
+                return f;
+            } catch (py::cast_error& e) {
+                throw std::runtime_error("The returned object of FactorType::new_factor is not a Factor.");
+            }
         }
 
         py::pybind11_fail("Tried to call pure virtual function \"FactorType::new_factor\"");
@@ -59,19 +64,26 @@ public:
 
     std::shared_ptr<Factor> new_factor(const ConditionalBayesianNetworkBase& model,
                                        const std::string& variable,
-                                       const std::vector<std::string>& parents) const override {
+                                       const std::vector<std::string>& parents,
+                                       py::args args = py::args{},
+                                       py::kwargs kwargs = py::kwargs{}) const override {
         pybind11::gil_scoped_acquire gil;
         pybind11::function override = pybind11::get_override(static_cast<const FactorType*>(this), "new_factor");
 
         if (override) {
-            auto o = override(model.shared_from_this(), variable, parents);
+            auto o = override(model.shared_from_this(), variable, parents, *args, **kwargs);
 
             if (o.is(py::none())) {
-                throw std::invalid_argument("FactorType::new_factor can not return None.");
+                throw std::invalid_argument("FactorType::new_factor cannot return None.");
             }
 
-            auto f = o.cast<std::shared_ptr<Factor>>();
-            return Factor::keep_python_alive(f);
+            try {
+                auto f = o.cast<std::shared_ptr<Factor>>();
+                Factor::keep_python_alive(f);
+                return f;
+            } catch (py::cast_error& e) {
+                throw std::runtime_error("The returned object of FactorType::new_factor is not a Factor.");
+            }
         }
 
         py::pybind11_fail("Tried to call pure virtual function \"FactorType::new_factor\"");
@@ -144,17 +156,17 @@ public:
             auto o = override();
 
             if (o.is(py::none())) {
-                throw std::invalid_argument("Factor::type can not return None.");
+                throw std::invalid_argument("Factor::type cannot return None.");
             }
 
-            auto type = o.get_type();
-            if (!is_factortype_subclass(type)) {
-                py::pybind11_fail("Returned object from \"Factor::type\" is not a subclass of \"FactorType\".");
+            try {
+                m_type = o.cast<std::shared_ptr<FactorType>>();
+                // Keep the type in the class member, so type_ref() can return a valid reference.
+                FactorType::keep_python_alive(m_type);
+                return m_type;
+            } catch (py::cast_error& e) {
+                throw std::runtime_error("The returned object of Factor::type is not a FactorType.");
             }
-
-            m_type = o.cast<std::shared_ptr<FactorType>>();
-            m_type = FactorType::keep_python_alive(m_type);
-            return m_type;
         }
 
         py::pybind11_fail("Tried to call pure virtual function \"Factor::type\"");
@@ -221,15 +233,85 @@ private:
 };
 
 void pybindings_factors(py::module& root) {
-    auto factors = root.def_submodule("factors", R"doc(The pybnesian.factors implements different types of factors.
-The factors are usually represented as conditional probability functions and are a component of a Bayesian network.
+    py::class_<Args>(root, "Args").def(py::init<py::args>(), R"doc(
+The :class:`Args` defines a wrapper over `*args`. This class allows to distinguish between a tuple representing `*args`
+or a tuple parameter while using :class:`Arguments <pybnesian.Arguments>`.
+
+.. _Example Args/Kwargs:
+
+Example:
+
+.. code-block:: python
+
+    Arguments({ 'a' : ((1, 2), {'param': 3}) })
+    # or
+    Arguments({ 'a' : Args((1, 2), {'param': 3}) })
+
+defines an `*args` with 2 arguments: a tuple (1, 2) and a dict {'param': 3}. No `**kwargs` is defined.
+
+.. code-block:: python
+
+    Arguments({ 'a' : (Args(1, 2), Kwargs(param = 3)) })
+
+defines an `*args` with 2 arguments: 1 and 2. It also defines a `**kwargs` with param = 3.
+)doc");
+    py::class_<Kwargs>(root, "Kwargs").def(py::init<py::kwargs>(), R"doc(
+The :class:`Kwargs` defines a wrapper over `**kwargs`. This class allows to distinguish between a dict representing
+`**kwargs` or a dict parameter while using :class:`Arguments <pybnesian.Arguments>`.
+
+See `Example Args/Kwargs`_.
+)doc");
+    py::class_<Arguments>(root, "Arguments", R"doc(
+The :class:`Arguments` class collects different arguments to construct :class:`Factor <pybnesian.Factor>`.
+
+The :class:`Arguments` object is constructed from a dictionary that associates each :class:`Factor <pybnesian.Factor>`
+configuration with a set of arguments.
+
+The keys of the dictionary can be:
+
+    - A 2-tuple (``name``, ``factor_type``) defines arguments for a :class:`Factor <pybnesian.Factor>` of variable
+      ``name`` with :class:`FactorType <pybnesian.FactorType>` ``factor_type``.
+    - An str defines arguments for a :class:`Factor <pybnesian.Factor>` of variable ``name``.
+    - A :class:`FactorType <pybnesian.FactorType>` defines arguments for a :class:`Factor <pybnesian.Factor>` with
+      :class:`FactorType <pybnesian.FactorType>` ``factor_type``.
+
+The values of the dictionary can be:
+
+    - A 2-tuple (:class:`Args <pybnesian.Args>`, :class:`Kwargs <pybnesian.Kwargs>`) defines `*args` and `**kwargs`.
+    - An :class:`Args <pybnesian.Args>` or tuple ( ... )  defines only `*args`.
+    - A :class:`Kwargs <pybnesian.Kwargs>` or dict { ... }: defines only `**kwargs`.
+
+When searching for the defined arguments in :class:`Arguments <pybnesian.Arguments>` for a given factor with ``name``
+and ``factor_type``, the most specific configurations have preference over more general ones.
+
+    - If a 2-tuple (``name``, ``factor_type``) configuration exists, the corresponding arguments are returned.
+    - Else, if a ``name`` configuration exists, the corresponding arguments are returned.
+    - Else, if a ``factor_type`` configuration exists, the corresponding arguments are returned.
+    - Else, empty `*args` and `**kwargs` are returned.
+)doc")
+        .def(py::init<>(), R"doc(
+Initializes an empty :class:`Arguments`.
+)doc")
+        .def(py::init<py::dict>(), py::arg("dict_arguments"), R"doc(
+Initializes a new :class:`Arguments` with the given configurations and arguments.
+
+:param dict_arguments: A dictionary { configurations : arguments} that associates each
+    :class:`Factor <pybnesian.Factor>` configuration with a set of arguments.
+)doc")
+        .def("__repr__", [](const Arguments&) { return "Arguments"; })
+        .def("args", &Arguments::args, py::arg("node"), py::arg("node_type"), R"doc(
+Returns the \*args and \*\*kwargs defined for a ``node`` with a given ``node_type``.
+
+:param node: A node name.
+:param node_type: :class:`FactorType <pybnesian.FactorType>` for ``node``.
+:returns: 2-tuple containing ``(*args, **kwargs)``
 )doc");
 
-    py::class_<FactorType, PyFactorType, std::shared_ptr<FactorType>> factor_type(factors, "FactorType", R"doc(
+    py::class_<FactorType, PyFactorType, std::shared_ptr<FactorType>> factor_type(root, "FactorType", R"doc(
 A representation of a :class:`Factor` type.
 )doc");
 
-    py::class_<Factor, PyFactor, std::shared_ptr<Factor>> factor(factors, "Factor");
+    py::class_<Factor, PyFactor, std::shared_ptr<Factor>> factor(root, "Factor");
 
     factor_type
         .def(py::init<>(), R"doc(Initializes a new :class:`FactorType`)doc")
@@ -250,6 +332,7 @@ A representation of a :class:`Factor` type.
         .def("__getstate__", [](const FactorType& self) { return self.__getstate__(); })
         // Setstate for pyderived type
         .def("__setstate__", [](py::object& self, py::tuple& t) { PyFactorType::__setstate__(self, t); })
+        .def("__hash__", &FactorType::hash)
         .def("__repr__", [](const FactorType& self) { return self.ToString(); })
         .def("__str__", [](const FactorType& self) { return self.ToString(); });
 
@@ -260,18 +343,23 @@ A representation of a :class:`Factor` type.
             .def("new_factor",
                  py::overload_cast<const ConditionalBayesianNetworkBase&,
                                    const std::string&,
-                                   const std::vector<std::string>&>(&FactorType::new_factor, py::const_),
+                                   const std::vector<std::string>&,
+                                   py::args,
+                                   py::kwargs>(&FactorType::new_factor, py::const_),
                  py::arg("model"),
                  py::arg("variable"),
                  py::arg("evidence"))
             .def("new_factor",
-                 py::overload_cast<const BayesianNetworkBase&, const std::string&, const std::vector<std::string>&>(
-                     &FactorType::new_factor, py::const_),
+                 py::overload_cast<const BayesianNetworkBase&,
+                                   const std::string&,
+                                   const std::vector<std::string>&,
+                                   py::args,
+                                   py::kwargs>(&FactorType::new_factor, py::const_),
                  py::arg("model"),
                  py::arg("variable"),
                  py::arg("evidence"),
                  R"doc(
-new_factor(self: pybnesian.factors.FactorType, model: BayesianNetworkBase or ConditionalBayesianNetworkBase, variable: str, evidence: List[str]) -> pybnesian.factors.Factor
+new_factor(self: pybnesian.FactorType, model: BayesianNetworkBase or ConditionalBayesianNetworkBase, variable: str, evidence: List[str], *args, **kwargs) -> pybnesian.Factor
 
 Create a new corresponding :class:`Factor` for a ``model`` with the given ``variable`` and ``evidence``.
 
@@ -280,6 +368,8 @@ Note that ``evidence`` might be different from ``model.parents(variable)``.
 :param model: The model that will contain the :class:`Factor`.
 :param variable: Variable name.
 :param evidence: List of evidence variable names.
+:param args: Additional arguments to construct the :class:`Factor`.
+:param kwargs: Additional keyword arguments used to construct the :class:`Factor`.
 :returns: A corresponding :class:`Factor` with the given ``variable`` and ``evidence``.
 )doc");
     }
@@ -361,7 +451,7 @@ Returns the sum of the log-likelihood of each instance in the DataFrame ``df``.
             py::arg("seed") = std::nullopt,
             R"doc(
 Samples ``n`` values from this :class:`Factor`. This method returns a :class:`pyarrow.Array` with ``n`` values with
-the same type returned by :func:``Factor.data_type``.
+the same type returned by :func:`Factor.data_type`.
 
 If this :class:`Factor` has evidence variables, the DataFrame ``evidence_values`` contains ``n`` instances for each
 evidence variable. Each sampled instance must be conditioned on ``evidence_values``.
@@ -380,9 +470,7 @@ Saves the :class:`Factor` in a pickle file with the given name.
         .def("__getstate__", [](const Factor& self) { return self.__getstate__(); })
         .def("__setstate__", [](py::object& self, py::tuple& t) { PyFactor::__setstate__(self, t); });
 
-    auto continuous = factors.def_submodule("continuous");
-
-    py::class_<UnknownFactorType, FactorType, std::shared_ptr<UnknownFactorType>>(factors, "UnknownFactorType", R"doc(
+    py::class_<UnknownFactorType, FactorType, std::shared_ptr<UnknownFactorType>>(root, "UnknownFactorType", R"doc(
 :class:`UnknownFactorType` is the representation of an unknown :class:`FactorType`. This factor type is assigned by
 default to each node in an heterogeneous Bayesian network.
 )doc")
@@ -393,7 +481,7 @@ Instantiates an :class:`UnknownFactorType`.
                         [](py::tuple&) { return UnknownFactorType::get(); }));
 
     py::class_<LinearGaussianCPDType, FactorType, std::shared_ptr<LinearGaussianCPDType>>(
-        continuous, "LinearGaussianCPDType", R"doc(
+        root, "LinearGaussianCPDType", R"doc(
 :class:`LinearGaussianCPDType` is the corresponding CPD type of :class:`LinearGaussianCPD`.
 )doc")
         .def(py::init(&LinearGaussianCPDType::get), R"doc(
@@ -402,7 +490,7 @@ Instantiates a :class:`LinearGaussianCPDType`.
         .def(py::pickle([](const LinearGaussianCPDType& self) { return self.__getstate__(); },
                         [](py::tuple&) { return LinearGaussianCPDType::get(); }));
 
-    py::class_<LinearGaussianCPD, Factor, std::shared_ptr<LinearGaussianCPD>>(continuous, "LinearGaussianCPD", R"doc(
+    py::class_<LinearGaussianCPD, Factor, std::shared_ptr<LinearGaussianCPD>>(root, "LinearGaussianCPD", R"doc(
 This is a linear Gaussian CPD:
 
 .. math::
@@ -421,7 +509,7 @@ It is parametrized by the following attributes:
 
 .. doctest::
 
-    >>> from pybnesian.factors.continuous import LinearGaussianCPD
+    >>> from pybnesian import LinearGaussianCPD
     >>> cpd = LinearGaussianCPD("a", ["b"])
     >>> assert not cpd.fitted()
     >>> cpd.beta
@@ -483,87 +571,7 @@ Returns the cumulative distribution function values of each instance in the Data
         .def(py::pickle([](const LinearGaussianCPD& self) { return self.__getstate__(); },
                         [](py::tuple t) { return LinearGaussianCPD::__setstate__(t); }));
 
-    py::class_<KDE>(continuous, "KDE", R"doc(
-This class implements Kernel Density Estimation (KDE) for a set of variables:
-
-.. math::
-
-    \hat{f}(\text{variables}) = \frac{1}{N\lvert\mathbf{H} \rvert} \sum_{i=1}^{N}
-    K(\mathbf{H}^{-1}(\text{variables} - \mathbf{t}_{i}))
-
-where :math:`N` is the number of training instances, :math:`K()` is the multivariate Gaussian kernel function,
-:math:`\mathbf{t}_{i}` is the :math:`i`-th training instance, and :math:`\mathbf{H}` is the bandwidth matrix.
-)doc")
-        .def(py::init<std::vector<std::string>>(), py::arg("variables"), R"doc(
-Initializes a KDE with the given ``variables``.
-
-:param variables: List of variable names.
-)doc")
-        .def("variables", &KDE::variables, R"doc(
-Gets the variable names:
-
-:returns: List of variable names.
-)doc")
-        .def("num_instances", &KDE::num_instances, R"doc(
-Gets the number of training instances (:math:`N`).
-
-:returns: Number of training instances.
-)doc")
-        .def("num_variables", &KDE::num_variables, R"doc(
-Gets the number of variables.
-
-:returns: Number of variables.
-)doc")
-        .def_property("bandwidth", &KDE::bandwidth, &KDE::setBandwidth, R"doc(
-Bandwidth matrix (:math:`\mathbf{H}`)
-
-)doc")
-        .def("dataset", &KDE::training_data, R"doc(
-Gets the training dataset for this KDE (the :math:`\mathbf{t}_{i}` instances).
-
-:returns: Training instance.
-)doc")
-        .def("fitted", &KDE::fitted, R"doc(
-Checks whether the model is fitted.
-
-:returns: True if the model is fitted, False otherwise.
-)doc")
-        .def("data_type", &KDE::data_type, R"doc(
-Returns the :class:`pyarrow.DataType` that represents the type of data handled by the :class:`KDE`.
-
-It can return :func:`pyarrow.float64` or :func:`pyarrow.float32`.
-
-:returns: the :class:`pyarrow.DataType` physical data type representation of the :class:`KDE`.
-)doc")
-        .def("fit", (void (KDE::*)(const DataFrame&)) & KDE::fit, py::arg("df"), R"doc(
-Fits the :class:`KDE` with the data in ``df``. It estimates the bandwidth :math:`\mathbf{H}` automatically using the
-Scott's rule [Scott]_.
-
-:param df: DataFrame to fit the :class:`KDE`.
-)doc")
-        .def("logl", &KDE::logl, py::return_value_policy::take_ownership, py::arg("df"), R"doc(
-Returns the log-likelihood of each instance in the DataFrame ``df``.
-
-:param df: DataFrame to compute the log-likelihood.
-:returns: A :class:`numpy.ndarray` vector with dtype :class:`numpy.float64`, where the i-th value is the log-likelihod
-          of the i-th instance of ``df``.
-)doc")
-        .def("slogl", &KDE::slogl, py::arg("df"), R"doc(
-Returns the sum of the log-likelihood of each instance in the DataFrame ``df``. That is, the sum of the result of
-:func:`KDE.slogl`.
-
-:param df: DataFrame to compute the sum of the log-likelihood.
-:returns: The sum of log-likelihood for DataFrame ``df``.
-)doc")
-        .def("save", &KDE::save, py::arg("filename"), R"doc(
-Saves the :class:`Factor` in a pickle file with the given name.
-
-:param filename: File name of the saved graph.
-)doc")
-        .def(py::pickle([](const KDE& self) { return self.__getstate__(); },
-                        [](py::tuple t) { return KDE::__setstate__(t); }));
-
-    py::class_<CKDEType, FactorType, std::shared_ptr<CKDEType>>(continuous, "CKDEType", R"doc(
+    py::class_<CKDEType, FactorType, std::shared_ptr<CKDEType>>(root, "CKDEType", R"doc(
 :class:`CKDEType` is the corresponding CPD type of :class:`CKDE`.
 )doc")
         .def(py::init(&CKDEType::get), R"doc(
@@ -572,7 +580,7 @@ Instantiates a :class:`CKDEType`.
         .def(py::pickle([](const CKDEType& self) { return self.__getstate__(); },
                         [](py::tuple&) { return CKDEType::get(); }));
 
-    py::class_<CKDE, Factor, std::shared_ptr<CKDE>>(continuous, "CKDE", R"doc(
+    py::class_<CKDE, Factor, std::shared_ptr<CKDE>>(root, "CKDE", R"doc(
 A conditional kernel density estimator (CKDE) is the ratio of two KDE models:
 
 .. math::
@@ -580,9 +588,9 @@ A conditional kernel density estimator (CKDE) is the ratio of two KDE models:
     \hat{f}(\text{variable} \mid \text{evidence}) =
     \frac{\hat{f}_{K}(\text{variable}, \text{evidence})}{\hat{f}_{K}(\text{evidence})}
 
-where \hat{f}_{K} is a :class:`KDE` estimation.
+where :math:`\hat{f}_{K}` is a :class:`KDE` estimation.
 )doc")
-        .def(py::init<const std::string, const std::vector<std::string>>(),
+        .def(py::init<std::string, std::vector<std::string>>(),
              py::arg("variable"),
              py::arg("evidence"),
              R"doc(
@@ -590,6 +598,21 @@ Initializes a new :class:`CKDE` with a given ``variable`` and ``evidence``.
 
 :param variable: Variable name.
 :param evidence: List of evidence variable names.
+)doc")
+        .def(py::init<>([](std::string variable,
+                           std::vector<std::string> evidence,
+                           std::shared_ptr<BandwidthSelector> bandwidth_selector) {
+                 return CKDE(variable, evidence, BandwidthSelector::keep_python_alive(bandwidth_selector));
+             }),
+             py::arg("variable"),
+             py::arg("evidence"),
+             py::arg("bandwidth_selector"),
+             R"doc(
+Initializes a new :class:`CKDE` with a given ``variable`` and ``evidence``.
+
+:param variable: Variable name.
+:param evidence: List of evidence variable names.
+:param bandwidth_selector: Procedure to fit the bandwidth.
 )doc")
         .def("num_instances", &CKDE::num_instances, R"doc(
 Gets the number of training instances (:math:`N`).
@@ -616,10 +639,7 @@ Returns the cumulative distribution function values of each instance in the Data
         .def(py::pickle([](const CKDE& self) { return self.__getstate__(); },
                         [](py::tuple t) { return CKDE::__setstate__(t); }));
 
-    auto discrete = factors.def_submodule("discrete");
-
-    py::class_<DiscreteFactorType, FactorType, std::shared_ptr<DiscreteFactorType>>(
-        discrete, "DiscreteFactorType", R"doc(
+    py::class_<DiscreteFactorType, FactorType, std::shared_ptr<DiscreteFactorType>>(root, "DiscreteFactorType", R"doc(
 :class:`DiscreteFactorType` is the corresponding CPD type of :class:`DiscreteFactor`.
 )doc")
         .def(py::init(&DiscreteFactorType::get), R"doc(
@@ -628,7 +648,7 @@ Instantiates a :class:`DiscreteFactorType`.
         .def(py::pickle([](const DiscreteFactorType& self) { return self.__getstate__(); },
                         [](py::tuple&) { return DiscreteFactorType::get(); }));
 
-    py::class_<DiscreteFactor, Factor, std::shared_ptr<DiscreteFactor>>(discrete, "DiscreteFactor", R"doc(
+    py::class_<DiscreteFactor, Factor, std::shared_ptr<DiscreteFactor>>(root, "DiscreteFactor", R"doc(
 This is a discrete factor implemented as a conditional probability table (CPT).
 )doc")
         .def(py::init<std::string, std::vector<std::string>>(), py::arg("variable"), py::arg("evidence"), R"doc(
@@ -640,11 +660,11 @@ Initializes a new :class:`DiscreteFactor` with a given ``variable`` and ``eviden
         .def(py::pickle([](const DiscreteFactor& self) { return self.__getstate__(); },
                         [](py::tuple t) { return DiscreteFactor::__setstate__(t); }));
 
-    py::class_<Assignment>(factors, "Assignment", R"doc(
-:class:`Assignment <pybnesian.factors.Assignment>` represents the assignment of values to a set of variables.
+    py::class_<Assignment>(root, "Assignment", R"doc(
+:class:`Assignment <pybnesian.Assignment>` represents the assignment of values to a set of variables.
 )doc")
         .def(py::init<std::unordered_map<std::string, AssignmentValue>>(), py::arg("assignments"), R"doc(
-Initializes an :class:`Assignment <pybnesian.factors.Assignment>` from a dict that contains the value for each variable.
+Initializes an :class:`Assignment <pybnesian.Assignment>` from a dict that contains the value for each variable.
 The key of the dict is the name of the variable, and the value of the dict can be an ``str`` or a ``float`` value.
 
 :param assignments: Value assignments for each variable.
@@ -662,19 +682,19 @@ Returns the assignment value for a given ``variable``.
             },
             py::arg("variables"),
             R"doc(
-Checks whether the :class:`Assignment <pybnesian.factors.Assignment>` contains assignments for all the ``variables``.
+Checks whether the :class:`Assignment <pybnesian.Assignment>` contains assignments for all the ``variables``.
 
 :param variables: Variable names.
-:returns: True if the :class:`Assignment <pybnesian.factors.Assignment>` contains values for all the given variables,
+:returns: True if the :class:`Assignment <pybnesian.Assignment>` contains values for all the given variables,
     False otherwise.
 )doc")
         .def("empty", &Assignment::empty, R"doc(
-Checks whether the :class:`Assignment <pybnesian.factors.Assignment>` does not have assignments.
+Checks whether the :class:`Assignment <pybnesian.Assignment>` does not have assignments.
 
-:returns: True if the :class:`Assignment <pybnesian.factors.Assignment>` does not have assignments, False otherwise.
+:returns: True if the :class:`Assignment <pybnesian.Assignment>` does not have assignments, False otherwise.
 )doc")
         .def("size", &Assignment::size, R"doc(
-Gets the number of assignments in the :class:`Assignment <pybnesian.factors.Assignment>`.
+Gets the number of assignments in the :class:`Assignment <pybnesian.Assignment>`.
 
 :returns: The number of assignments.
 )doc")
@@ -705,7 +725,7 @@ Removes the assignment for the ``variable``.
         .def(py::pickle([](const Assignment& self) { return self.__getstate__(); },
                         [](py::object& o) { return Assignment::__setstate__(o); }));
 
-    py::class_<CLinearGaussianCPD, Factor, std::shared_ptr<CLinearGaussianCPD>>(continuous, "CLinearGaussianCPD")
+    py::class_<CLinearGaussianCPD, Factor, std::shared_ptr<CLinearGaussianCPD>>(root, "CLinearGaussianCPD")
         .def(py::init<std::string, std::vector<std::string>>())
         .def(py::init<std::string, std::vector<std::string>, VectorXd, double>())
         .def(py::init<std::string,
@@ -715,9 +735,31 @@ Removes the assignment for the ``variable``.
         .def(py::pickle([](const CLinearGaussianCPD& self) { return self.__getstate__(); },
                         [](py::tuple t) { return CLinearGaussianCPD::__setstate__(t); }));
 
-    py::class_<DCKDE, Factor, std::shared_ptr<DCKDE>>(continuous, "DCKDE")
+    py::class_<HCKDE, Factor, std::shared_ptr<HCKDE>>(root, "HCKDE")
         .def(py::init<std::string, std::vector<std::string>>())
-        .def("conditional_factor", &DCKDE::conditional_factor, py::return_value_policy::reference_internal)
-        .def(py::pickle([](const DCKDE& self) { return self.__getstate__(); },
-                        [](py::tuple t) { return DCKDE::__setstate__(t); }));
+        .def(py::init<>([](std::string variable,
+                           std::vector<std::string> evidence,
+                           std::shared_ptr<BandwidthSelector> bandwidth_selector) {
+                 return HCKDE(variable, evidence, BandwidthSelector::keep_python_alive(bandwidth_selector));
+             }),
+             py::arg("variable"),
+             py::arg("evidence"),
+             py::arg("bandwidth_selector"))
+        .def(
+            py::init<>([](std::string variable,
+                          std::vector<std::string> evidence,
+                          std::unordered_map<Assignment, std::tuple<std::shared_ptr<BandwidthSelector>>, AssignmentHash>
+                              args) {
+                for (auto& arg : args) {
+                    BandwidthSelector::keep_python_alive(std::get<0>(arg.second));
+                }
+
+                return HCKDE(variable, evidence, args);
+            }),
+            py::arg("variable"),
+            py::arg("evidence"),
+            py::arg("bandwidth_selector"))
+        .def("conditional_factor", &HCKDE::conditional_factor, py::return_value_policy::reference_internal)
+        .def(py::pickle([](const HCKDE& self) { return self.__getstate__(); },
+                        [](py::tuple t) { return HCKDE::__setstate__(t); }));
 }
