@@ -58,10 +58,13 @@ An :class:`OperatorTabuSet` that contains forbidden operators.
         .def(py::init<>(), R"doc(
 Creates an empty :class:`OperatorTabuSet`.
 )doc")
-        .def("insert",
-             py::overload_cast<const std::shared_ptr<Operator>&>(&OperatorTabuSet::insert),
-             py::arg("operator"),
-             R"doc(
+        .def(
+            "insert",
+            [](OperatorTabuSet& self, const std::shared_ptr<Operator>& op) {
+                self.insert(Operator::keep_python_alive(op));
+            },
+            py::arg("operator"),
+            R"doc(
 Inserts an operator into the tabu set.
 
 :param operator: Operator to insert.
@@ -109,8 +112,12 @@ public:
 
         if (override) {
             auto o = override(m.shared_from_this());
-            auto nn = o.cast<std::vector<std::string>>();
-            return nn;
+
+            try {
+                return o.cast<std::vector<std::string>>();
+            } catch (py::cast_error& e) {
+                throw std::runtime_error("The returned object of Operator::nodes_changed is not a list of str.");
+            }
         }
 
         py::pybind11_fail("Tried to call pure virtual function \"Operator::nodes_changed\"");
@@ -122,8 +129,12 @@ public:
 
         if (override) {
             auto o = override(m.shared_from_this());
-            auto nn = o.cast<std::vector<std::string>>();
-            return nn;
+
+            try {
+                return o.cast<std::vector<std::string>>();
+            } catch (py::cast_error& e) {
+                throw std::runtime_error("The returned object of Operator::nodes_changed is not a list of str.");
+            }
         }
 
         py::pybind11_fail("Tried to call pure virtual function \"Operator::nodes_changed\"");
@@ -148,11 +159,16 @@ public:
                 auto o = override(m.shared_from_this());
 
                 if (o.is(py::none())) {
-                    throw std::invalid_argument("Operator::opposite can not return None.");
+                    throw std::invalid_argument("Operator::opposite cannot return None.");
                 }
 
-                auto op = o.cast<std::shared_ptr<Operator>>();
-                return Operator::keep_python_alive(op);
+                try {
+                    auto op = o.cast<std::shared_ptr<Operator>>();
+                    Operator::keep_python_alive(op);
+                    return op;
+                } catch (py::cast_error& e) {
+                    throw std::runtime_error("The returned object of Operator::opposite is not a Operator.");
+                }
             }
         }
 
@@ -179,11 +195,16 @@ public:
                 auto o = override(m.shared_from_this());
 
                 if (o.is(py::none())) {
-                    throw std::invalid_argument("Operator::opposite can not return None.");
+                    throw std::invalid_argument("Operator::opposite cannot return None.");
                 }
 
-                auto op = o.cast<std::shared_ptr<Operator>>();
-                return Operator::keep_python_alive(op);
+                try {
+                    auto op = o.cast<std::shared_ptr<Operator>>();
+                    Operator::keep_python_alive(op);
+                    return op;
+                } catch (py::cast_error& e) {
+                    throw std::runtime_error("The returned object of Operator::opposite is not a Operator.");
+                }
             }
         }
 
@@ -244,6 +265,8 @@ class PyOperatorSet : public OperatorSet {
 public:
     PyOperatorSet(bool calculate_local_score = true) : m_calculate_local_score(calculate_local_score) {}
 
+    bool is_python_derived() const override { return true; }
+
     void cache_scores(const BayesianNetworkBase& model, const Score& score) override {
         pybind11::gil_scoped_acquire gil;
         pybind11::function override = pybind11::get_override(static_cast<const OperatorSet*>(this), "cache_scores");
@@ -303,8 +326,13 @@ public:
             py::function override = pybind11::get_override(static_cast<const OperatorSet*>(this), "find_max");
             if (override) {
                 auto o = override(model.shared_from_this());
-                auto op = o.cast<std::shared_ptr<Operator>>();
-                return Operator::keep_python_alive(op);
+                try {
+                    auto op = o.cast<std::shared_ptr<Operator>>();
+                    Operator::keep_python_alive(op);
+                    return op;
+                } catch (py::cast_error& e) {
+                    throw std::runtime_error("The returned object of Operator::find_max is not a Operator.");
+                }
             }
         }
 
@@ -332,8 +360,13 @@ public:
             py::function override = pybind11::get_override(static_cast<const OperatorSet*>(this), "find_max");
             if (override) {
                 auto o = override(model.shared_from_this());
-                auto op = o.cast<std::shared_ptr<Operator>>();
-                return Operator::keep_python_alive(op);
+                try {
+                    auto op = o.cast<std::shared_ptr<Operator>>();
+                    Operator::keep_python_alive(op);
+                    return op;
+                } catch (py::cast_error& e) {
+                    throw std::runtime_error("The returned object of Operator::find_max is not a Operator.");
+                }
             }
         }
 
@@ -363,8 +396,13 @@ public:
             py::function override = pybind11::get_override(static_cast<const OperatorSet*>(this), "find_max_tabu");
             if (override) {
                 auto o = override(model.shared_from_this(), &tabu);
-                auto op = o.cast<std::shared_ptr<Operator>>();
-                return Operator::keep_python_alive(op);
+                try {
+                    auto op = o.cast<std::shared_ptr<Operator>>();
+                    Operator::keep_python_alive(op);
+                    return op;
+                } catch (py::cast_error& e) {
+                    throw std::runtime_error("The returned object of Operator::find_max_tabu is not a Operator.");
+                }
             }
         }
 
@@ -395,8 +433,13 @@ public:
             py::function override = pybind11::get_override(static_cast<const OperatorSet*>(this), "find_max_tabu");
             if (override) {
                 auto o = override(model.shared_from_this(), &tabu);
-                auto op = o.cast<std::shared_ptr<Operator>>();
-                return Operator::keep_python_alive(op);
+                try {
+                    auto op = o.cast<std::shared_ptr<Operator>>();
+                    Operator::keep_python_alive(op);
+                    return op;
+                } catch (py::cast_error& e) {
+                    throw std::runtime_error("The returned object of Operator::find_max_tabu is not a Operator.");
+                }
             }
         }
 
@@ -491,9 +534,7 @@ private:
 };
 
 void pybindings_operators(py::module& root) {
-    auto operators = root.def_submodule("operators", "Learning operators submodule");
-
-    py::class_<Operator, PyOperator<>, std::shared_ptr<Operator>> op(operators, "Operator", R"doc(
+    py::class_<Operator, PyOperator<>, std::shared_ptr<Operator>> op(root, "Operator", R"doc(
 An operator is the representation of a change in a Bayesian network structure. Each operator has a delta score
 associated that measures the difference in score when the operator is applied to the Bayesian network.
 )doc");
@@ -543,14 +584,13 @@ the same hash value.**
                  py::arg("model"),
                  py::return_value_policy::take_ownership,
                  R"doc(
-opposite(self: pybnesian.learning.operators.Operator, model: BayesianNetworkBase or ConditionalBayesianNetworkBase) -> Operator
+opposite(self: pybnesian.Operator, model: BayesianNetworkBase or ConditionalBayesianNetworkBase) -> Operator
 
 Returns an operator that reverses this :class:`Operator` given the ``model``. For example:
 
 .. doctest::
 
-    >>> from pybnesian.learning.operators import AddArc, RemoveArc
-    >>> from pybnesian.models import GaussianNetwork
+    >>> from pybnesian import AddArc, RemoveArc, GaussianNetwork
     >>> gbn = GaussianNetwork(["a", "b"])
     >>> add = AddArc("a", "b", 1)
     >>> assert add.opposite(gbn) == RemoveArc("a", "b", -1)
@@ -565,7 +605,7 @@ Returns an operator that reverses this :class:`Operator` given the ``model``. Fo
                  py::overload_cast<const ConditionalBayesianNetworkBase&>(&Operator::nodes_changed, py::const_),
                  py::arg("model"),
                  R"doc(
-nodes_changed(self: pybnesian.learning.operators.Operator, model: BayesianNetworkBase or ConditionalBayesianNetworkBase) -> List[str]
+nodes_changed(self: pybnesian.Operator, model: BayesianNetworkBase or ConditionalBayesianNetworkBase) -> List[str]
 
 Gets the list of nodes whose local score changes when the operator is applied.
 
@@ -573,7 +613,8 @@ Gets the list of nodes whose local score changes when the operator is applied.
 :returns: List of nodes whose local score changes when the operator is applied.
 )doc");
     }
-    py::class_<ArcOperator, Operator, PyArcOperator<>, std::shared_ptr<ArcOperator>>(operators, "ArcOperator", R"doc(
+
+    py::class_<ArcOperator, Operator, PyArcOperator<>, std::shared_ptr<ArcOperator>>(root, "ArcOperator", R"doc(
 This class implements an operator that perfoms a change in a single arc.
 )doc")
         .def(
@@ -595,12 +636,14 @@ Gets the target of the :class:`ArcOperator`.
 :returns: Name of the target node.
 )doc");
 
-    register_ArcOperators(operators);
+    register_ArcOperators(root);
 
-    py::class_<ChangeNodeType, Operator, std::shared_ptr<ChangeNodeType>>(operators, "ChangeNodeType", R"doc(
+    py::class_<ChangeNodeType, Operator, std::shared_ptr<ChangeNodeType>>(root, "ChangeNodeType", R"doc(
 This operator changes the :class:`FactorType` of a node.
 )doc")
-        .def(py::init<std::string, std::shared_ptr<FactorType>, double>(),
+        .def(py::init<>([](std::string node, std::shared_ptr<FactorType> node_type, double delta) {
+                 return ChangeNodeType(node, FactorType::keep_python_alive(node_type), delta);
+             }),
              py::arg("node"),
              py::arg("node_type"),
              py::arg("delta"),
@@ -622,9 +665,9 @@ Gets the new :class:`FactorType` of the :class:`ChangeNodeType`.
 :returns: New :class:`FactorType` of the node.
 )doc");
 
-    register_OperatorTabuSet(operators);
+    register_OperatorTabuSet(root);
 
-    py::class_<LocalScoreCache, std::shared_ptr<LocalScoreCache>>(operators, "LocalScoreCache", R"doc(
+    py::class_<LocalScoreCache, std::shared_ptr<LocalScoreCache>>(root, "LocalScoreCache", R"doc(
 This class implements a cache for the local score of each node.
 )doc")
         .def(py::init<>(), R"doc(
@@ -639,13 +682,13 @@ Initializes a :class:`LocalScoreCache` for the given ``model``.
 Caches the local score for all the nodes.
 
 :param model: A Bayesian network model.
-:param score: A :class:`Score <pybnesian.learning.scores.Score>` object to calculate the score.
+:param score: A :class:`Score <pybnesian.Score>` object to calculate the score.
 )doc")
         .def("cache_vlocal_scores", &LocalScoreCache::cache_vlocal_scores, py::arg("model"), py::arg("score"), R"doc(
 Caches the validation local score for all the nodes.
 
 :param model: A Bayesian network model.
-:param score: A :class:`ValidatedScore <pybnesian.learning.scores.ValidatedScore>` object to calculate the score.
+:param score: A :class:`ValidatedScore <pybnesian.ValidatedScore>` object to calculate the score.
 )doc")
         .def("update_local_score",
              &LocalScoreCache::update_local_score,
@@ -656,7 +699,7 @@ Caches the validation local score for all the nodes.
 Updates the local score of the ``node`` in the ``model``.
 
 :param model: A Bayesian network model.
-:param score: A :class:`Score <pybnesian.learning.scores.Score>` object to calculate the score.
+:param score: A :class:`Score <pybnesian.Score>` object to calculate the score.
 :param node: A node name.
 )doc")
         .def("update_vlocal_score",
@@ -668,7 +711,7 @@ Updates the local score of the ``node`` in the ``model``.
 Updates the validation local score of the ``node`` in the ``model``.
 
 :param model: A Bayesian network model.
-:param score: A :class:`ValidatedScore <pybnesian.learning.scores.ValidatedScore>` object to calculate the score.
+:param score: A :class:`ValidatedScore <pybnesian.ValidatedScore>` object to calculate the score.
 :param node: A node name.
 )doc")
         .def("sum", &LocalScoreCache::sum, R"doc(
@@ -684,7 +727,7 @@ Returns the local score of the ``node`` in the ``model``.
 :returns: Local score of ``node`` in ``model``.
 )doc");
 
-    py::class_<OperatorSet, PyOperatorSet, std::shared_ptr<OperatorSet>>(operators, "OperatorSet", R"doc(
+    py::class_<OperatorSet, PyOperatorSet, std::shared_ptr<OperatorSet>>(root, "OperatorSet", R"doc(
 The :class:`OperatorSet` coordinates a set of operators. It caches/updates the score of each operator in the set and
 finds the operator with the best score.
 )doc")
@@ -715,7 +758,7 @@ Returns the current :class:`LocalScoreCache` of this :class:`OperatorSet`.
 Caches the delta score values of each operator in the set.
 
 :param model: Bayesian network model.
-:param score: The :class:`Score <pybnesian.learning.scores.Score>` object to cache the scores.
+:param score: The :class:`Score <pybnesian.Score>` object to cache the scores.
 )doc")
         .def(
             "find_max",
@@ -763,14 +806,14 @@ Updates the delta score values of the operators in the set after applying an ope
 ``changed_nodes`` determines the nodes whose local score has changed after applying the operator.
 
 :param model: Bayesian network model.
-:param score: The :class:`Score <pybnesian.learning.scores.Score>` object to cache the scores.
+:param score: The :class:`Score <pybnesian.Score>` object to cache the scores.
 :param changed_nodes: The nodes whose local score has changed.
 )doc")
         .def("set_arc_blacklist",
              py::overload_cast<const ArcStringVector&>(&OperatorSet::set_arc_blacklist),
              py::arg("arc_blacklist"),
              R"doc(
-Sets the arc blacklist (a list of arcs that can not be added).
+Sets the arc blacklist (a list of arcs that cannot be added).
 
 :param arc_blacklist: The list of blacklisted arcs.
 )doc")
@@ -787,12 +830,24 @@ Sets the max indegree allowed. This may change the set of valid operators.
 
 :param max_indegree: Max indegree allowed.
 )doc")
-        .def("set_type_blacklist", &OperatorSet::set_type_blacklist, py::arg("type_blacklist"), R"doc(
+        .def(
+            "set_type_blacklist",
+            [](OperatorSet& self, const FactorTypeVector& type_blacklist) {
+                self.set_type_blacklist(util::keep_FactorTypeVector_python_alive(type_blacklist));
+            },
+            py::arg("type_blacklist"),
+            R"doc(
 Sets the type blacklist (a list of :class:`FactorType` that are not allowed).
 
 :param type_blacklist: The list of blacklisted :class:`FactorType`.
 )doc")
-        .def("set_type_whitelist", &OperatorSet::set_type_whitelist, py::arg("type_whitelist"), R"doc(
+        .def(
+            "set_type_whitelist",
+            [](OperatorSet& self, const FactorTypeVector& type_whitelist) {
+                self.set_type_whitelist(util::keep_FactorTypeVector_python_alive(type_whitelist));
+            },
+            py::arg("type_whitelist"),
+            R"doc(
 Sets the type whitelist (a list of :class:`FactorType` that are forced).
 
 :param type_whitelist: The list of whitelisted :class:`FactorType`.
@@ -802,7 +857,7 @@ Marks the finalization of the algorithm. It clears the state of the object, so
 :func:`OperatorSet.cache_scores` can be called again.
 )doc");
 
-    py::class_<ArcOperatorSet, OperatorSet, std::shared_ptr<ArcOperatorSet>>(operators, "ArcOperatorSet", R"doc(
+    py::class_<ArcOperatorSet, OperatorSet, std::shared_ptr<ArcOperatorSet>>(root, "ArcOperatorSet", R"doc(
 This set of operators contains all the operators related with arc changes (:class:`AddArc`, :class:`RemoveArc`,
 :class:`FlipArc`)
 )doc")
@@ -818,11 +873,13 @@ Initializes an :class:`ArcOperatorSet` with optional sets of arc blacklists/whit
 :param max_indegree: Max indegree allowed.
 )doc");
 
-    py::class_<ChangeNodeTypeSet, OperatorSet, std::shared_ptr<ChangeNodeTypeSet>>(
-        operators, "ChangeNodeTypeSet", R"doc(
+    py::class_<ChangeNodeTypeSet, OperatorSet, std::shared_ptr<ChangeNodeTypeSet>>(root, "ChangeNodeTypeSet", R"doc(
 This set of operators contains all the possible operators of type :class:`ChangeNodeType`.
 )doc")
-        .def(py::init<FactorTypeVector, FactorTypeVector>(),
+        .def(py::init<>([](FactorTypeVector type_blacklist, FactorTypeVector type_whitelist) {
+                 return ChangeNodeTypeSet(util::keep_FactorTypeVector_python_alive(type_blacklist),
+                                          util::keep_FactorTypeVector_python_alive(type_whitelist));
+             }),
              py::arg("type_blacklist") = FactorTypeVector(),
              py::arg("type_whitelist") = FactorTypeVector(),
              R"doc(
@@ -832,10 +889,14 @@ Initializes a :class:`ChangeNodeTypeSet` with blacklisted and whitelisted :class
 :param type_whitelist: The list of whitelisted :class:`FactorType`.
 )doc");
 
-    py::class_<OperatorPool, OperatorSet, std::shared_ptr<OperatorPool>>(operators, "OperatorPool", R"doc(
+    py::class_<OperatorPool, OperatorSet, std::shared_ptr<OperatorPool>>(root, "OperatorPool", R"doc(
 This set of operators can join a list of :class:`OperatorSet`, so that they can act as a single :class:`OperatorSet`.
 )doc")
-        .def(py::init<std::vector<std::shared_ptr<OperatorSet>>>(), py::arg("opsets"), R"doc(
+        .def(py::init<>([](std::vector<std::shared_ptr<OperatorSet>> opsets) {
+                 return OperatorPool(OperatorSet::keep_vector_python_alive(opsets));
+             }),
+             py::arg("opsets"),
+             R"doc(
 Initializes an :class:`OperatorPool` with a list of :class:`OperatorSet`.
 
 :param opsets: List of :class:`OperatorSet`.
